/*********************************************************************
    This file is part of Siworin. Copyright © 2021 by Ruud Harmsen.

    Siworin is a simple word indexer and local search engine. See
    https://rudhar.com/sfreview/siworin/#siw04 and
    https://rudhar.com/sfreview/siworin/toolsrc/siworin .

    Siworin is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    Siworin is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Siworin.  If not, see <https://www.gnu.org/licenses/>.
  ********************************************************************/

#ifndef _SIWORIN_CONFIG_H
#define _SIWORIN_CONFIG_H

enum siworin_configindex_e
{
/* The maximum number of occurrences of a word for inclusion in
   the index. If there are too many, searching becomes trivial,
   so the word is suppressed.
   See also https://rudhar.com/sfreview/siworin/siworin11.htm

   Note: = 0 is the initialiser of the enum, not the default
   value of the config setting! See function init_config in
   source file siworin-config.c for defaults. */
   SIWORIN_CONFIG_INDEX_WORD_MAX_OCCUR = 0,

/* Added 24 september 2022:
   Write an informative file to see how frequent words are.
   Default: don't do it, as it isn't necessary */
   SIWORIN_CONFIG_WRITE_WORDS_FREQ,

/* An optional directory path (default: current directory), and
   the first part of the filenames, before the dot. Example and
   default: /var/www/cgi-bin/siworin/siworin, then the files are
   var/www/cgi-bin/siworin/siworin.words etc., see also
   siworin-files.c . */
   SIWORIN_CONFIG_INDEX_FILES_PREFIX,

/* Two strings to edit file paths and turn them into valid URL,
   accessible via the browser.
   Example: replace
   /var/www/html/ by
   https://rudhar.com/ */
   SIWORIN_FILEPATH_REPLACE_THIS,
   SIWORIN_FILEPATH_REPLACE_BY,

/* Path and filenames for the html of the search page. */
   SIWORIN_HEAD_HTML_FILE,

/* See also https://rudhar.com/sfreview/siworin/siworin10.htm#id20210919 */
   SIWORIN_MINIM_MATCH,
   SIWORIN_MAXIM_DISTA,
   SIWORIN_SHOW_CONTEXT_BEFOR,
   SIWORIN_SHOW_CONTEXT_AFTER,

/* Search engine siworin is meant primarily to search HTML- and XML-like
   material. Then removing tags (what is between < and >) is useful.
   But siworin also works on e-mail files, e.g. in the mbx format used
   by Eudora. Because of the convention to start quoted lines with >,
   tag removal then has undesirable effects. Therefore it can be disabled,
   by setting this config switch to 1. The default is 0, i.e., do remove
   them.
   There is a separate switch for word separation (disabling will also
   index e-mail addresses between < and >), and context when displaying
   a result. */
   SIWORIN_DONT_REMOVE_TAGS_WORDSEP,
   SIWORIN_DONT_REMOVE_TAGS_CONTEXT,

/* Support only alphabetic words (the default), or also 'words' that
   begin with a digit, and/or also words that have digits in them
   further on. */
   SIWORIN_INCLUDE_DIGITS_INITIAL,
   SIWORIN_INCLUDE_DIGITS_SUBSEQU,

/* Ranking parameters */

/* Having more matching words is better than fewer.
   This factor is added to the ranking for each matching word. */
   SIWORIN_RANKING_NUMWORDS,

/* Matching a rare word is better than matching a frequent word.
   This factor is divided by how often the word is in the index,
   and the result added to the ranking. */
   SIWORIN_RANKING_WORDFREQ,

/* Matching words that are close together are better than more distant
   words. This factor is divided by the distance in bytes between the
   matches, and the result added to the ranking. */
   SIWORIN_RANKING_WORDDIST,

/* Sometimes there are very many searchresult, most of them hardly
   relevant. Many browsers take a long time loading them. It is
   often better to limit the number of results to display. */
   SIWORIN_MAX_RESULTS_TO_DISPLAY,
};

/* Searches a program's invocation parameters, passed as argv.
   As soon as any of
   -cconfigpath
   -c configpath
   --config configpath
   is found, siworin_set_explicit_config_path is called with
   configpath.

   The function returns the number of command line arguments
   interpreted, that is, 1 or 2, or zero if nothing was found.
 */
int siworin_args_to_config_path (char **argv);

/* Sets path for the siworon config file. If it exists,
   it takes precedence over all default paths to find
   a config file.

   If no explicit config file path is set, or it is set but
   the file cannot be opened, attempts will be made to find
   a config file as follows, in this order:
   "./siworin.conf",
   "/usr/local/siworin.conf",
   "/usr/local/siworin/siworin.conf",
   "/var/www/cgi-bin/siworin.conf",
   "/var/www/cgi-bin/siworin/siworin.conf",

   As soon as a config file has been opened succesfully,
   all other paths, existent or not, will be ignored. If
   no config file is found at all, default config values
   apply.
*/
void siworin_set_explicit_config_path (char *path);
char *siworin_get_explicit_config_path (void);

char *siworin_config_value (enum siworin_configindex_e index);

/* Free all malloc'ed memory areas (including realloc and calloc)
   internally allocated by siworin-config.c */
void siworin_config_freeall (void);

#endif
